#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "etslib.h"

void tighten(char *p)
{
  char *e;
  while (p[0] != 0)
    {
      e = strchr(p, 0) - 1;
      
      if (*e == ' ' || *e == '\n' || *e == '\r' || (unsigned char) *e == 0xFF)
	*e = 0;
      else
	break;
    }
}

void print_entry(char *rmsn, char *emsn, char *imsn, int o, int s, int day, int month, int smin, int dsec)
{
  char *f;

  if (o)
    f = "%02i.%02i %02i:%02i %3i:%2i %s %s -> %s -> %s\n";
  else
    f = "%02i.%02i %02i:%02i %3i:%2i %s %s <- %s <- %s\n";

  printf(f,
	 day,
	 month,
	 smin / 60,
	 smin % 60,
	 dsec / 60,
	 dsec % 60,

	 s ? "+" : "-", 

	 *imsn == 0 ? "? " : imsn,
	 *emsn == 0 ? "? " : emsn,
	 *rmsn == 0 ? "? " : rmsn);
}


void parse(char *p)
{
  char remote_msn[22];
  char external_msn[3];
  char internal_msn[3];
  char flags[4];
  char day[3];
  char month[3];
  char hour[3];
  char minutes[3];

  char duration_minutes[3];
  char duration_seconds[3];

  
  if (strlen(p) != 73)
    {
      fprintf(stderr, "broken line. %i\n", strlen(p));
      return;
    }

  bzero(remote_msn, sizeof(remote_msn));
  strncpy(remote_msn, p, sizeof(remote_msn)-1);

  bzero(external_msn, sizeof(external_msn));
  strncpy(external_msn, &p[25], sizeof(external_msn)-1);

  bzero(internal_msn, sizeof(internal_msn));
  strncpy(internal_msn, &p[32], sizeof(internal_msn)-1);

  bzero(flags, sizeof(flags));
  strncpy(flags, &p[35], sizeof(flags)-1);

  bzero(day, sizeof(day));
  strncpy(day, &p[39], sizeof(day)-1);

  bzero(month, sizeof(month));
  strncpy(month, &p[42], sizeof(month)-1);

  bzero(hour, sizeof(hour));
  strncpy(hour, &p[45], sizeof(hour)-1);

  bzero(minutes, sizeof(minutes));
  strncpy(minutes, &p[48], sizeof(minutes)-1);

  bzero(duration_minutes, sizeof(duration_minutes));
  strncpy(duration_minutes, &p[51], sizeof(duration_minutes)-1);

  bzero(duration_seconds, sizeof(duration_seconds));
  strncpy(duration_seconds, &p[55], sizeof(duration_seconds)-1);
  
  tighten(remote_msn);

  if (remote_msn[0] == '*') remote_msn[0] = 0;
  if (external_msn[0] == 'x') external_msn[0] = 0;
  if (internal_msn[0] == 'T') internal_msn[0] = 0;

  print_entry(remote_msn, external_msn, internal_msn, flags[0] == 'G' ? 1 : 0, flags[2] == 'V' ? 0 : 1, atoi(day), atoi(month), atoi(hour)*60+atoi(minutes), atoi(duration_minutes)*60+atoi(duration_seconds));
}
     


int main(int argc, char*argv[])
{
  int fd;
  struct termios t;
  
  if (argc != 2)
    return 1;

  if (strcmp(argv[1], "-") == 0)
    fd = 0;
  else
    fd = tty_open(argv[1], &t);

  if (fd < 0) 
    return 2;

  tty_write_byte(fd, '$');
  tty_write_byte(fd, '%');      
  tty_write_byte(fd, '%');   
  tty_write_byte(fd, '%');   
  tty_write_byte(fd, '%');      
  
  while (1)
    {
      char *p;


      p = tty_read_line(fd);

      if (!p)
	break;
      
      tty_chomp(p);

      if (strcmp(p, "") == 0)
	{
	  free(p);
	  break;
	}

      parse(p);
      //printf("%s\n", p);
      free(p);

      tty_write_byte(fd, '%');
    }
  

  tty_close(fd, &t);

  return 0;
}

  
