#include <stdio.h>
#include <unistd.h>
#include <sys/stat.h>
#include <limits.h>
#include <fcntl.h>
#include <stdlib.h>

#include "etslib.h"

int channel_fifo[3] = {-1, -1, -1};

int open_channel_fifos()
{
  int c;
  for (c = 0 ; c < 3; c++)
    {
      char fn[PATH_MAX];
      struct stat sstat;
      int fd, n;

      snprintf(fn, sizeof(fn), "/dev/isdnctrl%i", c+7);


      if (stat(fn, &sstat) == 0)
	{
	  if (! S_ISFIFO(sstat.st_mode))
	    {
	      fprintf(stderr, "File %s does exist, but is not a fifo. Aborting.\n", fn);
	      return 1;
	    }
	}
      else
	{
	  fprintf(stderr, "Fifo %s does not exist. Trying to create.\n", fn);

	  if (mkfifo(fn, S_IRUSR|S_IWUSR) != 0)
	    {
	      fprintf(stderr, "Could not create fifo '%s'. Aborting.\n", fn);
	      return 1;
	    }
	}

      fd = open(fn, O_RDWR|O_NDELAY);
      if (fd < 0)
	{
	  fprintf(stderr, "Could not open fifo '%s'. Aborting.\n", fn);
	  return 1;
	}

      //      n = fcntl(fd, F_GETFL, 0);
      //fcntl(fd, F_SETFL, n & ~O_NDELAY);

      channel_fifo[c] = fd;
    }

  return 0;
}

void close_channel_fifos()
{
  int c;

  for (c = 0; c < 3; c++)
    {
      if (channel_fifo[c] >= 0)
	close(channel_fifo[c]);
      
      channel_fifo[c] = -1;
    }
}

void work (unsigned char*p, int l)
{
  int i, channel;

  if (l <= 3)
    {
      fprintf(stderr, "Warning: Too short message recieved, ignoring. (%i)\n", l);
      return;
    }
  
  if (p[0] != 0x0A)
    {
      fprintf(stderr, "Warning: Unknown type of message recieved, ignoring. (%i)\n", p[0]);
      return;
    }
  
  channel = p[3];
  if (channel < 0 || channel > 2)
    {
      fprintf(stderr, "Warning: Message from unknown S0-bus recieved, ignoring. (%i)\n", p[3]);
      return;
    }


  // printf("HEX:");

  if (write(channel_fifo[channel], "HEX:", 4) != 4)
    return;
 
  for (i = 4; i < l; i++)
    if (!(p[i] == 0xFF && p[i+1] == 0xFF))
      {
	// printf(" %02X", p[i]);
	char h[4];
	snprintf(h, sizeof(h), " %02X", p[i]);
	if (write(channel_fifo[channel], h, 3) != 3)
	  return;
      }
  
  //printf("\n");
  write(channel_fifo[channel], "\n", 1);
  
  //fflush(stdout);
}

int main(int argc, char*argv[])
{
  int fd;
  struct termios t;
  unsigned char buf[1024];
  unsigned int pos;
  int ff;
  
  if (argc != 2)
    return 1;

  if (strcmp(argv[1], "-") == 0)
    fd = 0;
  else
    fd = tty_open(argv[1], &t);

  if (fd < 0) 
    return 2;

  if (open_channel_fifos() != 0)
    {
      tty_close(fd, &t);
      return 3;
    }

  fprintf(stderr, "Initialisation ok.\n");
  
  pos = 0;
  ff = 0;

  while (1)
    {
      int c;

      c = read(fd, &buf[pos], 1);

      if (c != 1) 
	break;

      if (ff)
	{
	  if (buf[pos] == 0xFF)
	    {
	      pos--;
	    }
	  else
	    {
	      work(buf, pos-1);

	      buf[0] = buf[pos];
	      pos = 1;
	    }
	  
	  ff = 0;
	}
      
      if (buf[pos] == 0xFF)
	ff = 1;

      pos++;
      if (pos > sizeof(buf))
	{
	  fprintf(stderr, "BUFFER OVERFLOW!\n");
	  break;
	}
    }

  if (fd != 0)
    tty_close(fd, &t);

  close_channel_fifos();


  return 0;
}
