#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "etslib.h"

void dump(unsigned char *d, int size)
{
  int i;
  for (i = 0; i < size; i++)
    if (d[i] > 32 && d[i] < 127)
      printf("%c ", d[i]);
    else
      printf("0x%02X ", d[i]);
}

int main(int argc, char*argv[])
{
  int fd1, fd2;
  struct termios saved_termios1, saved_termios2;
  int quit;
  int last;

  if (argc != 3)
    {
      fprintf(stderr, "You have to specify 2 devices.\n");
      return 1;
    }

  fd1 = tty_open(argv[1], &saved_termios1);
  if (fd1 < 0)
    {
      fprintf(stderr, "Could not open device #1.\n");
      return 3;
    }
  
  fd2 = tty_open(argv[2], &saved_termios2);
  if (fd2 < 0)
    {
      fprintf(stderr, "Could not open device #2.\n");
      tty_close(fd1, &saved_termios1);
      return 4;
    }

  quit = 0;
  last = -1;

  while (!quit)
    {
      fd_set fds;
      int sr;
  
      FD_ZERO(&fds);
      FD_SET(fd1, &fds);
      FD_SET(fd2, &fds);
      
      sr = select(FD_SETSIZE, &fds, 0, 0, 0);

      if (sr <= 0)
	{
	  fprintf(stderr, "select() returned error.\n");
	  
	  tty_close(fd1, &saved_termios1);
	  tty_close(fd2, &saved_termios2);
	  return 5;
	}
      
      if (FD_ISSET(fd1, &fds))
	{
	  unsigned char buf[4096];
	  int l = read(fd1, buf, sizeof(buf));

	  if (l <= 0) 
	    {
	      fprintf(stderr, "\nBounced out on fd1. Reopening\n");
	      tty_close(fd1, &saved_termios1);
	      
	      fd1 = tty_open(argv[1], &saved_termios1);
	      if (fd1 < 0)
		{
		  fprintf(stderr, "Could not open device #1.\n");
		  return 3;
		}
	    }
	  else
	    {
	      write (fd2, buf, l);

	      if (last != 1)
		{
		  printf("\n");
		  printf("\nFD1->FD2: ");
		  last = 1;
		}

	      dump(buf, l);
	      fflush(stdout);

	    }
	}

      if (FD_ISSET(fd2, &fds))
	{
	  unsigned char buf[4096];
	  int l = read(fd2, buf, sizeof(buf));
	  
	  if (l <= 0) 
	    {
	      fprintf(stderr, "\nBounced out on fd2. Reopening\n");
	      tty_close(fd2, &saved_termios2);
	      
	      fd2 = tty_open(argv[1], &saved_termios2);
	      if (fd2 < 0)
		{
		  fprintf(stderr, "Could not open device #2.\n");
		  return 3;
		}
	    }
	  else
	    {
	      write (fd1, buf, l);

	      if (last != 2)
		{
		  printf("\n");
		  printf("FD2->FD1: ");
		  last = 2;
		}

	      dump(buf, l);
	      fflush(stdout);
		  
	    }
	}

    }
      
  tty_close(fd1, &saved_termios1);
  tty_close(fd2, &saved_termios2);

  return 0;
}
